
# q-expansion of newform 8030.2.a.h, downloaded from the LMFDB on 28 April 2024.

# We generate the q-expansion using the Hecke eigenvalues a_p at the primes.
# Each a_p is given as a linear combination
# of the following basis for the coefficient ring.

def make_data():

    from sage.all import prod, floor, prime_powers, gcd, QQ, primes_first_n, next_prime, RR

    def discrete_log(elts, gens, mod):
        # algorithm 2.2, page 16 of https://arxiv.org/abs/0903.2785
        def table_gens(gens, mod):
            T = [1]
            n = len(gens)
            r = [None]*n
            s = [None]*n
            for i in range(n):
                beta = gens[i]
                r[i] = 1
                N = len(T)
                while beta not in T:
                    for Tj in T[:N]:
                        T.append((beta*Tj) % mod)
                    beta = (beta*gens[i]) % mod
                    r[i] += 1
                s[i] = T.index(beta)
            return T, r, s
        T, r, s = table_gens(gens, mod)
        n = len(gens)
        N = [ prod(r[:j]) for j in range(n) ]
        Z = lambda s: [ (floor(s/N[j]) % r[j]) for j in range(n)]
        return [Z(T.index(elt % mod)) for elt in elts]
    def extend_multiplicatively(an):
        for pp in prime_powers(len(an)-1):
            for k in range(1, (len(an) - 1)//pp + 1):
                if gcd(k, pp) == 1:
                    an[pp*k] = an[pp]*an[k]
    from sage.all import PolynomialRing, NumberField
    R = PolynomialRing(QQ, "x")
    f = R(poly_data)
    K = NumberField(f, "a")
    betas = [K.gens()[0]**i for i in range(len(poly_data))]
    convert_elt_to_field = lambda elt: sum(c*beta for c, beta in zip(elt, betas))
    # convert aps to K elements
    primes = primes_first_n(len(aps_data))
    good_primes = [p for p in primes if not p.divides(level)]
    aps = map(convert_elt_to_field, aps_data)
    if not hecke_ring_character_values:
        # trivial character
        char_values = dict(zip(good_primes, [1]*len(good_primes)))
    else:
        gens = [elt[0] for elt in hecke_ring_character_values]
        gens_values = [convert_elt_to_field(elt[1]) for elt in hecke_ring_character_values]
        char_values = dict([(
            p,prod(g**k for g, k in zip(gens_values, elt)))
            for p, elt in zip(good_primes, discrete_log(good_primes, gens, level))
            ])
    an_list_bound = next_prime(primes[-1])
    an = [0]*an_list_bound
    an[1] = 1
    
    from sage.all import PowerSeriesRing
    PS = PowerSeriesRing(K, "q")
    for p, ap in zip(primes, aps):
        if p.divides(level):
            euler_factor = [1, -ap]
        else:
            euler_factor = [1, -ap, p**(weight - 1) * char_values[p]]
        k = RR(an_list_bound).log(p).floor() + 1
        foo = (1/PS(euler_factor)).padded_list(k)
        for i in range(1, k):
            an[p**i] = foo[i]
    extend_multiplicatively(an)
    return PS(an)
level = 8030
weight = 2
poly_data = [0, 1]

# The basis for the coefficient ring is just the power basis
# in the root of the defining polynomial above.
hecke_ring_character_values = None
aps_data = [[1], [-2], [-1], [4], [-1], [2], [-2], [-2], [4], [-6], [0], [8], [-6], [4], [6], [6], [-4], [8], [-2], [-8], [-1], [-16], [-4], [6], [14], [14], [2], [4], [20], [-4], [0], [4], [-10], [-12], [0], [10], [6], [20], [16], [-22], [-20], [-2], [-20], [-2], [-6], [-4], [-6], [0], [28], [26], [-6], [30], [4], [28], [18], [-4], [-10], [2], [22], [0], [24], [-26], [12], [32], [-20], [-4], [-20], [6], [32], [32], [34], [-4], [-4], [10], [28], [28], [26], [-16], [2], [24], [4], [18], [6], [-4], [20], [-4], [26], [-10], [-12], [-8], [-12], [24], [16], [-20], [4], [0], [-38], [-2], [-20], [14], [8], [26], [4], [-8], [4], [4], [38], [-22], [12], [-8], [-32], [6], [48], [0], [-44], [-50], [28], [-10], [0], [2], [-10], [14], [6], [-12], [36], [-42], [10], [-24], [-16], [10], [34], [-48], [-28], [28], [-36], [24], [18], [12], [-32], [-42], [42], [28], [-22], [12], [-50], [0], [18], [-26], [-36], [38], [-14], [2], [24], [-32], [-32], [48], [30], [-34], [-34], [-40], [18], [-34], [-32], [-28], [-12], [-42], [8], [-14], [24], [-48], [-54], [26], [-20], [-30], [32], [10], [-16], [6], [56], [-34], [64], [30], [-52], [30], [-16], [-6], [-26], [16], [-16], [-10], [32], [36], [-56], [62], [24], [-58], [40], [-22], [30], [-56], [0], [10], [36], [26], [-16], [-40], [-16], [36], [-18], [-40], [50], [70], [-54], [-8], [-10], [-10], [-44], [-36], [-6], [-22], [-16], [0], [2], [-60], [68], [-54], [34], [-24], [-28], [66], [6], [42], [-14], [-32], [-26], [48], [-42], [2], [-4], [28], [-44], [2], [34], [-2], [-46], [-22], [-34], [-28], [38], [56], [-26], [40], [-52], [58], [50], [-56], [-26], [4], [42], [58], [-64], [-32], [-38], [-46], [-26], [28], [-10], [38], [36], [70], [-10], [-4], [-34], [54], [-26], [42], [-38], [-44], [-22], [36], [0], [32], [-84], [-8], [-36], [24], [6], [-42], [32], [42], [-42], [-38], [-22], [44], [6], [-42], [-48], [-80], [62], [48], [18], [-62], [-26], [34], [20], [-82], [36], [34], [24], [-14], [72], [-42], [30], [68], [-8], [12], [36], [-12], [-82], [30], [-12], [68], [20], [82], [26], [30], [4], [4], [8], [-62], [-58], [-50], [8], [-44], [-70], [-18], [10], [2], [86], [-40], [-50], [52], [16], [-64], [-68], [70], [-70], [-56], [26], [58], [66], [70], [-14], [64], [-70], [42], [-48], [-30], [22], [46], [-26], [-42], [8], [2], [32], [46], [18], [92], [-56], [-52], [52], [-48], [-48], [-32], [100], [-8], [4], [82], [-14], [-94], [6], [0], [80], [-90], [8], [-80], [-40], [4], [4], [10], [-82], [-72], [60], [10], [-36], [-58], [20], [6], [-52], [58], [-8], [-24], [-8], [-78], [34], [-42], [44], [86], [4], [-36], [-22], [82], [-6], [50], [-26], [18], [60], [78], [70], [84], [2], [-50], [-2], [86], [52], [-76], [56], [18], [28], [-40]]
